@@ -39,6 +39,7 @@
 #include "gdk/gdk.h"
 #include "gdk/gdkkeysyms.h"
 
+#include "gtkalias.h"
 #include "gtkfontsel.h"
 
 #include "gtkbutton.h"
@@ -576,7 +577,7 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
 }
 
 GtkWidget *
-gtk_font_selection_new ()
+gtk_font_selection_new (void)
 {
   GtkFontSelection *fontsel;
   
@@ -643,7 +644,7 @@ set_cursor_to_iter (GtkTreeView *view,
   GtkTreeModel *model = gtk_tree_view_get_model (view);
   GtkTreePath *path = gtk_tree_model_get_path (model, iter);
   
-  gtk_tree_view_set_cursor (view, path, 0, FALSE);
+  gtk_tree_view_set_cursor (view, path, NULL, FALSE);
 
   gtk_tree_path_free (path);
 }
@@ -1370,6 +1371,13 @@ gtk_font_selection_dialog_init (GtkFontSelectionDialog *fontseldiag)
                                                   GTK_RESPONSE_OK);
   gtk_widget_grab_default (fontseldiag->ok_button);
   
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (fontseldiag),
+					   GTK_RESPONSE_OK,
+					   GTK_RESPONSE_APPLY,
+					   GTK_RESPONSE_CANCEL,
+					   -1);
+
+
   gtk_window_set_title (GTK_WINDOW (fontseldiag),
                         _("Font Selection"));
 
