@@ -37,6 +37,7 @@
 #endif /* G_OS_WIN32 */
 
 #include "gtkicontheme.h"
+#include "gtkiconfactory.h"
 #include "gtkiconcache.h"
 #include "gtkintl.h"
 #include "gtksettings.h"
@@ -567,7 +568,7 @@ gtk_icon_theme_init (GtkIconTheme *icon_theme)
   xdg_data_dirs = g_get_system_data_dirs ();
   for (i = 0; xdg_data_dirs[i]; i++) ;
 
-  priv->search_path_len = i + 3;
+  priv->search_path_len = 2 * i + 2;
   
   priv->search_path = g_new (char *, priv->search_path_len);
   
@@ -578,7 +579,8 @@ gtk_icon_theme_init (GtkIconTheme *icon_theme)
   for (j = 0; xdg_data_dirs[j]; j++) 
     priv->search_path[i++] = g_build_filename (xdg_data_dirs[j], "icons", NULL);
 
-  priv->search_path[i++] = g_strdup ("/usr/share/pixmaps");
+  for (j = 0; xdg_data_dirs[j]; j++) 
+    priv->search_path[i++] = g_build_filename (xdg_data_dirs[j], "pixmaps", NULL);
 
   priv->themes_valid = FALSE;
   priv->themes = NULL;
@@ -1895,7 +1897,12 @@ theme_lookup_icon (IconTheme          *theme,
       icon_info->cp_filename = g_locale_from_utf8 (icon_info->filename,
 						   -1, NULL, NULL, NULL);
 #endif
-      if (min_dir->cache && has_icon_file)
+      
+      if (min_dir->icon_data != NULL)
+	icon_info->data = g_hash_table_lookup (min_dir->icon_data, icon_name);
+      
+      if (icon_info->data == NULL &&
+	  min_dir->cache && has_icon_file)
 	{
 	  gchar *icon_file_name, *icon_file_path;
 
@@ -1908,13 +1915,12 @@ theme_lookup_icon (IconTheme          *theme,
 		min_dir->icon_data = g_hash_table_new_full (g_str_hash, g_str_equal,
 							    g_free, (GDestroyNotify)icon_data_free);
 	      load_icon_data (min_dir, icon_file_path, icon_file_name);
+	      
+	      icon_info->data = g_hash_table_lookup (min_dir->icon_data, icon_name);
 	    }
 	  g_free (icon_file_name);
 	  g_free (icon_file_path);
 	}
-      
-      if (min_dir->icon_data != NULL)
-	icon_info->data = g_hash_table_lookup (min_dir->icon_data, icon_name);
 
       icon_info->dir_type = min_dir->type;
       icon_info->dir_size = min_dir->size;
@@ -2867,6 +2873,8 @@ find_builtin_icon (const gchar *icon_name,
   gboolean has_larger = FALSE;
   BuiltinIcon *min_icon = NULL;
   
+  _gtk_icon_factory_ensure_default_icons ();
+  
   if (!icon_theme_builtin_icons)
     return NULL;
 
