@@ -30,6 +30,7 @@
 
 #include <config.h>
 
+#include "gtkalias.h"
 #include "gtkradioaction.h"
 #include "gtkradiomenuitem.h"
 #include "gtktoggleactionprivate.h"
@@ -53,7 +54,8 @@ enum
 enum 
 {
   PROP_0,
-  PROP_VALUE
+  PROP_VALUE,
+  PROP_GROUP
 };
 
 static void gtk_radio_action_init         (GtkRadioAction *action);
@@ -134,13 +136,28 @@ gtk_radio_action_class_init (GtkRadioActionClass *klass)
   g_object_class_install_property (gobject_class,
 				   PROP_VALUE,
 				   g_param_spec_int ("value",
-						     _("The value"),
-						     _("The value returned by gtk_radio_action_get_current_value() when this action is the current action of its group."),
+						     P_("The value"),
+						     P_("The value returned by gtk_radio_action_get_current_value() when this action is the current action of its group."),
 						     G_MININT,
 						     G_MAXINT,
 						     0,
 						     G_PARAM_READWRITE));
 
+  /**
+   * GtkRadioAction:group:
+   *
+   * Sets a new group for a radio action.
+   *
+   * Since: 2.4
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_GROUP,
+				   g_param_spec_object ("group",
+							P_("Group"),
+							P_("The radio action whose group this action belongs to."),
+							GTK_TYPE_RADIO_ACTION,
+							G_PARAM_WRITABLE));
+
   /**
    * GtkRadioAction::changed:
    * @action: the action on which the signal is emitted
@@ -171,6 +188,43 @@ gtk_radio_action_init (GtkRadioAction *action)
   action->private_data->value = 0;
 }
 
+/**
+ * gtk_radio_action_new:
+ * @name: A unique name for the action
+ * @label: The label displayed in menu items and on buttons
+ * @tooltip: A tooltip for this action
+ * @stock_id: The stock icon to display in widgets representing this action
+ * @value: The value which gtk_radio_action_get_current_value() should return
+ *    if this action is selected.
+ *
+ * Creates a new #GtkRadioAction object. To add the action to
+ * a #GtkActionGroup and set the accelerator for the action,
+ * call gtk_action_group_add_action_with_accel().
+ *
+ * Return value: a new #GtkRadioAction
+ *
+ * Since: 2.4
+ */
+GtkRadioAction *
+gtk_radio_action_new (const gchar *name,
+		      const gchar *label,
+		      const gchar *tooltip,
+		      const gchar *stock_id,
+		      gint value)
+{
+  GtkRadioAction *action;
+
+  action = g_object_new (GTK_TYPE_RADIO_ACTION,
+			 "name", name,
+			 "label", label,
+			 "tooltip", tooltip,
+			 "stock_id", stock_id,
+			 "value", value,
+			 NULL);
+
+  return action;
+}
+
 static void
 gtk_radio_action_finalize (GObject *object)
 {
@@ -191,8 +245,7 @@ gtk_radio_action_finalize (GObject *object)
       tmp_action->private_data->group = action->private_data->group;
     }
 
-  if (parent_class->finalize)
-    (* parent_class->finalize) (object);
+  (* parent_class->finalize) (object);
 }
 
 static void
@@ -210,6 +263,20 @@ gtk_radio_action_set_property (GObject         *object,
     case PROP_VALUE:
       radio_action->private_data->value = g_value_get_int (value);
       break;
+    case PROP_GROUP: 
+      {
+	GtkRadioAction *arg;
+	GSList *slist = NULL;
+	
+	if (G_VALUE_HOLDS_OBJECT (value)) 
+	  {
+	    arg = GTK_RADIO_ACTION (g_value_get_object (value));
+	    if (arg)
+	      slist = gtk_radio_action_get_group (arg);
+	    gtk_radio_action_set_group (radio_action, slist);
+	  }
+      }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -276,7 +343,7 @@ gtk_radio_action_activate (GtkAction *action)
 
 	  if (tmp_action->private_data->active && (tmp_action != toggle_action))
 	    {
-	      gtk_action_activate (GTK_ACTION (tmp_action));
+	      _gtk_action_emit_activate (GTK_ACTION (tmp_action));
 	      break;
 	    }
 	}
@@ -373,7 +440,7 @@ gtk_radio_action_set_group (GtkRadioAction *action,
  * gtk_radio_action_get_current_value:
  * @action: a #GtkRadioAction
  * 
- * Obtains the value property of the the currently active member of 
+ * Obtains the value property of the currently active member of 
  * the group to which @action belongs.
  * 
  * Return value: The value of the currently active group member
